﻿using System;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Collections.Generic;

namespace tune2
{
    public partial class Form1 : Form
    {
        int ostype = 0;
        String cwd = " ";
        myQueue rxqueue = new myQueue();
        int suchval = 0;
        int tbmhz_hasfocus = 0;
        bool anttrx = false;
        revcalc_proc rp = new revcalc_proc();
        double rpres_C = 0, rpres_L = 0, rpres_R = 0;
        int readcheck = 0;

        public Form1()
        {
            InitializeComponent();

            // test OS type
            OperatingSystem osversion = System.Environment.OSVersion;
            if (osversion.VersionString.Contains("indow"))
                ostype = 0;
            else
                ostype = 1;

            cwd = Directory.GetCurrentDirectory();

            readConfig();
            fillportnames();
            try
            {
                suchval = (int)(MyToDouble(cb_swr_suche.Text) * 100);
            }
            catch
            {
                suchval = 0;
            }

            rp.setValues(21.074, 23.95+0.6, 162+25, true);
        }

        List <String> comportlist = new List<string>();
        int comportlistidx = 0;

        void fillportnames()
        {
            string[] SerialPortNames = System.IO.Ports.SerialPort.GetPortNames();

            foreach (String s in SerialPortNames)
            {
                if (!comportlist.Contains(s) && (s.Contains("USB") || s.Contains("COM")))
                {
                    comportlist.Add(s);
                }
            }
        }

        bool serialopen = false;
        String serport;
        bool connected = false;
        DateTime lastConnReqTime = DateTime.UtcNow;

        void auto_openserial()
        {
            // readcheck überwacht als Watchdog die serielle Schnittstelle
            // wenn nichts mehr kommt, so wird sie neu geöffnet
            // das ist notwendig, da in Linux andere Programme die gleiche Schnittstelle öffnen können
            // und dann hier nichts mehr gehen würde
            if(readcheck > 1)
            {
                //Console.WriteLine("SERCHECK: " + readcheck.ToString());
            }
            if (!serialPort1.IsOpen || readcheck >= 5)
            {
                readcheck = 0;
                connected = false;
                serialopen = false;
                status_info.Text = "offline";
                status_info.ForeColor = Color.Red;
            }

            if (connected)
                return;

            lastconnected = false;

            fillportnames();
            if (comportlist.Count == 0)
            {
                status_info.Text = "keine ser.Ports";
                status_info.ForeColor = Color.Red;
                return;
            }

            // Versuche es nicht zu oft, lasse Pausen
            TimeSpan ts = DateTime.UtcNow - lastConnReqTime;
            if (ts.TotalMilliseconds < 500)
                return;

            try
            {
                if (serialPort1.IsOpen)
                {
                    try
                    {
                        serialPort1.Close();
                    }
                    catch { }
                }

                if (!comportlist.Contains(serport))
                {
                    serport = comportlist[0];
                }
                serialPort1.PortName = serport;
                serialPort1.Open();
                serialopen = true;
                status_info.Text = serialPort1.PortName + " offen";
                status_info.ForeColor = Color.Orange;
                connected = false;
                lastConnReqTime = DateTime.UtcNow;
                sendtuner2("HH", false); // melde dich beim Tuner
            }
            catch (Exception ex)
            {
                //MessageBox.Show(GetText(3), GetText(4), MessageBoxButtons.OK, MessageBoxIcon.Error);
                //MessageBox.Show(ex.ToString());
                serialopen = false;
                status_info.Text = "offline";
                status_info.ForeColor = Color.Red;
            }
            // als allererstes nehme serport auf den Config
            // danach scanne die Liste durch
            if (++comportlistidx >= comportlist.Count)
                comportlistidx = 0;

            serport = comportlist[comportlistidx];
        }

        public String myFilePath(String file)
        {
            if (ostype == 0)
            {
                // Windows
                return Application.UserAppDataPath + "\\" + file;
            }
            else
            {
                return cwd + "/" + file;
            }
        }


        private void saveConfig()
        {
            StreamWriter sw = null;
            try
            {
                Console.WriteLine("write: " + myFilePath("tune2.cfg"));
                using (sw = new StreamWriter(myFilePath("tune2.cfg")))
                {
                    sw.WriteLine(serport);
                    sw.WriteLine(cb_swr_suche.Text);
                    sw.WriteLine(cb_speed.Text);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(myFilePath("tune2.cfg") + " kann nicht geschrieben werden: " + e.ToString());
            }

        }

        private void readConfig()
        {
            StreamReader sr = null;
            try
            {
                Console.WriteLine("read: " + myFilePath("tune2.cfg"));
                using (sr = new StreamReader(myFilePath("tune2.cfg")))
                {
                    serport = ReadString(sr);

                    cb_swr_suche.Text = ReadString(sr);
                    if (cb_swr_suche.Text.Length < 2)
                        cb_swr_suche.Text = "1.1";

                    cb_speed.Text = ReadString(sr);
                    if (cb_speed.Text.Length < 2)
                        cb_speed.Text = "25";
                }
            }
            catch
            {
                Console.WriteLine(myFilePath("tune2.cfg") + " kann nicht gelesen werden");
            }
        }

        private String ReadString(StreamReader sr)
        {
            String s = sr.ReadLine();
            if (s != null)
            {
                return s;
            }
            return " ";
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            saveConfig();
        }

        UInt16 reg16 = 0xffff;        // Schieberegister

        UInt16 crc16_bytecalc(Byte byt)
        {
            int i;
            UInt16 polynom = 0x8408;      // Generatorpolynom

            for (i = 0; i < 8; ++i)
            {
                if ((reg16 & 1) != (byt & 1))
                    reg16 = (UInt16)((reg16 >> 1) ^ polynom);
                else
                    reg16 >>= 1;
                byt >>= 1;
            }
            return reg16;           // inverses Ergebnis, MSB zuerst
        }

        UInt16 crc16_messagecalc(Byte[] data, int len)
        {
            int i;

            reg16 = 0xffff;                 // Initialisiere Shift-Register mit Startwert
            for (i = 0; i < len; i++)
            {
                reg16 = crc16_bytecalc(data[i]);        // Berechne fuer jeweils 8 Bit der Nachricht
            }
            return reg16;
        }

        private void sendtuner2(Byte b1, Byte b2, Byte b3)
        {
            Byte[] tx = new Byte[3];
            tx[0] = b1;
            tx[1] = b2;
            tx[2] = b3;
            sendtuner2(tx);
        }

        private void sendtuner2(Byte [] b)
        {
            int idx = 0;
            Byte[] tx = new Byte[20];
            tx[idx++] = 0x45;
            tx[idx++] = 0x85;
            tx[idx++] = 0xf4;
            tx[idx++] = 0xa7;
            tx[idx++] = (Byte)(b.Length >> 8);
            tx[idx++] = (Byte)(b.Length & 0xff);
            for (int i=0; i<b.Length; i++)
                tx[idx++] = b[i];

            UInt16 crcval = crc16_messagecalc(b, b.Length);
            tx[idx++] = (Byte)((crcval >> 8) & 0xff);
            tx[idx++] = (Byte)(crcval & 0xff);

            try
            {
                serialPort1.Write(tx, 0, idx);
            }
            catch (Exception ex)
            {
                serialopen = false;
            }
        }

        private void sendtuner2(String mode, Byte [] b)
        {
            Byte[] tx = new Byte[b.Length + 2];
            tx[0] = (Byte)mode.ToCharArray()[0];
            tx[1] = (Byte)mode.ToCharArray()[1];
            for (int i = 0; i < b.Length; i++)
                tx[2 + i] = b[i];

            sendtuner2(tx);
        }

        private void sendtuner2(String mode, bool action)
        {
            sendtuner2((Byte)mode.ToCharArray()[0], (Byte)mode.ToCharArray()[1], (Byte)(action ? 1 : 0));
            sendinfo.Text = "TX:" + mode + (action ? "1" : "0");
        }

        void sendtuner2(String mode, int value)
        {
            Byte vl = (Byte)(value & 0xff);
            Byte vh = (Byte)(value >> 8);

            sendtuner2((Byte)mode.ToCharArray()[0],vh,vl);
            sendinfo.Text = "TX:" + mode + value.ToString();
        }

        void sendtuner2(String mode, Byte value)
        {
            sendtuner2((Byte)mode.ToCharArray()[0], (Byte)mode.ToCharArray()[1], value);
            sendinfo.Text = "TX:" + mode + value.ToString();
        }

        private void cb_C_alloff_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_C_alloff.Checked) sendtuner2("CA",false);
            cb_C_alloff.Checked = false;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked) sendtuner2("CE", false);
            checkBox1.Checked = false;
        }

        private void cb_C_1_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_C_1.Checked) sendtuner2("C1", cb_C_1.Checked);
            cb_C_1.Checked = false;
        }

        private void cb_C_2_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_C_2.Checked) sendtuner2("C2", cb_C_2.Checked);
            cb_C_2.Checked = false;
        }

        private void cb_C_3_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_C_3.Checked) sendtuner2("C3", cb_C_3.Checked);
            cb_C_3.Checked = false;
        }

        private void cb_C_4_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_C_4.Checked) sendtuner2("C4", cb_C_4.Checked);
            cb_C_4.Checked = false;
        }

        private void cb_C_5_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_C_5.Checked) sendtuner2("C5", cb_C_5.Checked);
            cb_C_5.Checked = false;
        }

        private void cb_C_6_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_C_6.Checked) sendtuner2("C6", cb_C_6.Checked);
            cb_C_6.Checked = false;
        }

        private void cb_C_7_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_C_7.Checked) sendtuner2("C7", cb_C_7.Checked);
            cb_C_7.Checked = false;
        }

        private void cb_C_8_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_C_8.Checked) sendtuner2("C8", cb_C_8.Checked);
            cb_C_8.Checked = false;
        }

        private void cb_L_alloff_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_L_alloff.Checked) sendtuner2("LA", false);
            cb_L_alloff.Checked = false;
        }

        private void cb_L_allon_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_allon.Checked) sendtuner2("LE", false);
            cb_L_allon.Checked = false;
        }

        private void cb_L_minus_Click(object sender, EventArgs e)
        {
            Int16 v = 0;
            if (cb_L_1.ForeColor == Color.Red) v |= 1;
            if (cb_L_2.ForeColor == Color.Red) v |= 2;
            if (cb_L_3.ForeColor == Color.Red) v |= 4;
            if (cb_L_4.ForeColor == Color.Red) v |= 8;
            if (cb_L_5.ForeColor == Color.Red) v |= 16;
            if (cb_L_6.ForeColor == Color.Red) v |= 32;
            if (cb_L_7.ForeColor == Color.Red) v |= 64;
            if (cb_L_8.ForeColor == Color.Red) v |= 128;
            if (cb_L_9.ForeColor == Color.Red) v |= 256;

            if (v > 0) v--;

            sendtuner2("I", v);
        }

        private void cb_L_plus_Click(object sender, EventArgs e)
        {
            Int16 v = 0;
            if (cb_L_1.ForeColor == Color.Red) v |= 1;
            if (cb_L_2.ForeColor == Color.Red) v |= 2;
            if (cb_L_3.ForeColor == Color.Red) v |= 4;
            if (cb_L_4.ForeColor == Color.Red) v |= 8;
            if (cb_L_5.ForeColor == Color.Red) v |= 16;
            if (cb_L_6.ForeColor == Color.Red) v |= 32;
            if (cb_L_7.ForeColor == Color.Red) v |= 64;
            if (cb_L_8.ForeColor == Color.Red) v |= 128;
            if (cb_L_9.ForeColor == Color.Red) v |= 256;

            if (v < 511) v++;

            sendtuner2("I", v);
        }

        private void cb_L_1_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_1.Checked) sendtuner2("L1", cb_L_1.Checked);
            cb_L_1.Checked = false;
        }

        private void cb_L_2_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_2.Checked) sendtuner2("L2", cb_L_2.Checked);
            cb_L_2.Checked = false;
        }

        private void cb_L_3_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_3.Checked) sendtuner2("L3", cb_L_3.Checked);
            cb_L_3.Checked = false;
        }

        private void cb_L_4_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_4.Checked) sendtuner2("L4", cb_L_4.Checked);
            cb_L_4.Checked = false;
        }

        private void cb_L_5_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_5.Checked) sendtuner2("L5", cb_L_5.Checked);
            cb_L_5.Checked = false;
        }

        private void cb_L_6_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_6.Checked) sendtuner2("L6", cb_L_6.Checked);
            cb_L_6.Checked = false;
        }

        private void cb_L_7_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_7.Checked) sendtuner2("L7", cb_L_7.Checked);
            cb_L_7.Checked = false;
        }

        private void cb_L_8_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_8.Checked) sendtuner2("L8", cb_L_8.Checked);
            cb_L_8.Checked = false;
        }

        private void cb_L_9_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_L_9.Checked) sendtuner2("L9", cb_L_9.Checked);
            cb_L_9.Checked = false;
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            sendtuner2("MO", false);
            checkBox2.Checked = false;
        }

        private void cb_L_trx_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_L_trx.Checked) sendtuner2("MT", cb_L_trx.Checked);
            cb_L_trx.Checked = false;
        }

        private void Cb_L_ant_CheckedChanged(object sender, EventArgs e)
        {
            if(cb_L_ant.Checked) sendtuner2("MA", cb_L_ant.Checked);
            cb_L_ant.Checked = false;
        }

        // freq in MHz
        int getFreqIdx(double freq)
        {
            freq *= 1e6;
            int freq_kHz = Convert.ToInt32(freq / 1000);
            int freq_5kHz = freq_kHz / 5;
            return freq_5kHz;
        }

        int cap,ind,iswr,ipwr;
        double fswr,fpwr,find,fantU,fantI;
        UInt32 civ_freq;
        bool Lant = false, Ltrx = false;
        const int relanz = 19;
        int[] relais = new int[relanz];
        bool lastconnected = false;
        int oldcap = -1, oldind = -1;
        bool oldanttrx = false;

        private void timer1_Tick(object sender, EventArgs e)
        {
            auto_openserial();

            // das Lesen der ser. Schnittstelle im INT geht unter Linux nicht, daher hier im Timer
            if (serialopen)
            {
                try
                {
                    while (serialPort1.BytesToRead > 0)
                    {
                        int ret = serialPort1.ReadByte();
                        if (ret == -1) break;
                        DataRec(ret);
                        if (got == 1)
                        {
                            readcheck = 0;
                            got = 0;
                            connected = true;
                            if (lastconnected == false)
                            {
                                status_info.Text = serialPort1.PortName + " verbunden";
                                status_info.ForeColor = Color.DarkGreen;
                                lastconnected = true;
                            }

                            receiveinfo.Text = "RX:" + type.ToString();

                            if (type == 0)
                            {
                                showProtocol();
                                return;
                            }

                            if (type == 1)
                            {
                                // Memory Belegung
                                for (int i = 0; i < 10; i++)
                                {
                                    Button bt = bt_mem_160;
                                    String btxt = "160m";
                                    switch (i)
                                    {
                                        case 1: bt = bt_mem_80; btxt = "80m"; break;
                                        case 2: bt = bt_mem_60; btxt = "60m"; break;
                                        case 3: bt = bt_mem_40; btxt = "40m"; break;
                                        case 4: bt = bt_mem_30; btxt = "30m"; break;
                                        case 5: bt = bt_mem_20; btxt = "20m"; break;
                                        case 6: bt = bt_mem_17; btxt = "17m"; break;
                                        case 7: bt = bt_mem_15; btxt = "15m"; break;
                                        case 8: bt = bt_mem_12; btxt = "12m"; break;
                                        case 9: bt = bt_mem_10; btxt = "10m"; break;
                                    }

                                    bt.Text = btxt + ": " + rxarr[i].ToString() + "%";

                                    if (rxarr[i] == 0) bt.BackColor = Color.Red;
                                    else if (rxarr[i] == 100) bt.BackColor = Color.Green;
                                    else bt.BackColor = Color.Orange;
                                }
                            }

                            if (type == 2)
                            {
                                // Messwerte
                                int idx = 0;

                                int ant_U = rxarr[idx++];
                                ant_U <<= 8;
                                ant_U += rxarr[idx++];
                                fantU = ant_U;
                                fantU /= 10.0;

                                int ant_I = rxarr[idx++];
                                ant_I <<= 8;
                                ant_I += rxarr[idx++];
                                fantI = ant_I;
                                fantI /= 100.0;

                                int tuningsource = rxarr[idx++]; // 1=vom mem

                                int z = rxarr[idx++];
                                int phi = rxarr[idx++];
                                int ant = rxarr[idx++];

                                cap = rxarr[idx++];
                                cap <<= 8;
                                cap += rxarr[idx++];

                                ind = rxarr[idx++];
                                ind <<= 8;
                                ind += rxarr[idx++];
                                find = ind;
                                find /= 1000;

                                iswr = rxarr[idx++];
                                iswr <<= 8;
                                iswr += rxarr[idx++];
                                fswr = iswr;
                                fswr /= 100.0;

                                ipwr = rxarr[idx++];
                                ipwr <<= 8;
                                ipwr += rxarr[idx++];
                                fpwr = ipwr;
                                fpwr /= 10.0;

                                civ_freq = rxarr[idx++];
                                civ_freq <<= 8;
                                civ_freq += rxarr[idx++];
                                civ_freq <<= 8;
                                civ_freq += rxarr[idx++];
                                civ_freq <<= 8;
                                civ_freq += rxarr[idx++];

                                if (tbmhz_hasfocus < 99)
                                {
                                    if (tbmhz_hasfocus > 0)
                                        tbmhz_hasfocus--;

                                    if (tbmhz_hasfocus == 0)
                                        tb_mhz.Text = (Convert.ToDouble(civ_freq) / 1e6).ToString();
                                }

                                Byte civ_adr = rxarr[idx++];
                                Byte civ_request = rxarr[idx++];
                                anttrx = (rxarr[idx++] == 0 ? true : false);

                                uint tuned_freq = rxarr[idx++];
                                tuned_freq <<= 8;
                                tuned_freq += rxarr[idx++];
                                tuned_freq <<= 8;
                                tuned_freq += rxarr[idx++];
                                tuned_freq <<= 8;
                                tuned_freq += rxarr[idx++];
                                tb_tunedToFreq.Text = (Convert.ToDouble(tuned_freq) / 1e6).ToString("n3") + " MHz";

                                int itemp = rxarr[idx++];
                                tb_TEMP.Text = itemp.ToString();

                                int UB = rxarr[idx++];
                                UB <<= 8;
                                UB += rxarr[idx++];
                                tb_UB.Text = ((double)(UB / 1000)).ToString("n1");

                                rp.setValues(civ_freq/1e6, ind/1e3 + 0.35, cap + 25, anttrx);

                                if (civ_freq > 0)
                                {
                                    tb_civfrequ.Text = (Convert.ToDouble(civ_freq) / 1e6).ToString("n6") + " MHz";

                                    if (civ_request == 0)
                                    {
                                        status_info1.ForeColor = Color.Blue;
                                        status_info1.Text = "      CIV (DSP-7)";
                                    }
                                    else
                                    {
                                        status_info1.ForeColor = Color.DarkGreen;
                                        status_info1.Text = "      CIV (Tuner)";
                                    }

                                    
                                }
                                else
                                {
                                    tb_civfrequ.Text = "Freq: ?";
                                    status_info1.Text = "no CIV";
                                    status_info1.ForeColor = Color.Red;
                                }

                                if (iswr == 0)
                                {
                                    lb_lcstatus.Text = "RX";
                                }
                                else
                                {
                                    if (fswr > 99) fswr = 99;
                                    lb_lcstatus.Text = "SWR = 1 : " + fswr.ToString("n2").PadLeft(3);
                                }
                                tb_GesamtC.Text = cap.ToString() + "pF";
                                tb_GesamtL.Text = find.ToString("n3") + "uH";
                                tb_storeinfo.Text = "QRG: " + tb_civfrequ.Text + "\n\r" + "C: " + tb_GesamtC.Text + "\n\r" + "L: " + tb_GesamtL.Text;
                                tb_Z.Text = (z < 255) ? z.ToString() : "?";
                                tb_PHI.Text = (phi == 1) ? "+" : "-";
                                tb_tuningsource.Text = (tuningsource == 1) ? "MEM" : "TUN";
                                tb_ant_u.Text = fantU.ToString("n1");
                                tb_ant_i.Text = fantI.ToString("n1");
                                tb_ant_P.Text = (fantU * fantI).ToString("n0");
                                tb_ant_pwr.Text = fpwr.ToString("n0");
                                rb_ant1.ForeColor = Color.Green;
                                rb_ant2.ForeColor = Color.Green;
                                rb_ant3.ForeColor = Color.Green;
                                rb_ant4.ForeColor = Color.Green;
                                rb_ant5.ForeColor = Color.Green;
                                rb_ant6.ForeColor = Color.Green;
                                switch (ant)
                                {
                                    case 0: rb_ant1.ForeColor = Color.Red; break;
                                    case 1: rb_ant2.ForeColor = Color.Red; break;
                                    case 2: rb_ant3.ForeColor = Color.Red; break;
                                    case 3: rb_ant4.ForeColor = Color.Red; break;
                                    case 4: rb_ant5.ForeColor = Color.Red; break;
                                    case 5: rb_ant6.ForeColor = Color.Red; break;
                                }
                                if (!civ_has_focus)
                                    tb_civadr.Text = civ_adr.ToString("X2");

                                if (cap != oldcap || ind != oldind || oldanttrx != anttrx)
                                {
                                    oldcap = cap;
                                    oldind = ind;
                                    oldanttrx = anttrx;
                                    pictureBox_diagramm.Invalidate();
                                }
                            }

                            if (type == 3)
                            {
                                // Relaisposition
                                for (int i = 0; i < relanz; i++)
                                {
                                    relais[i] = rxarr[i];
                                }

                                Ltrx = (relais[0] == 1);
                                Lant = (relais[1] == 1);
                                cb_L_trx.ForeColor = (relais[0] == 1 ? Color.Red : Color.Green);
                                cb_L_ant.ForeColor = (relais[1] == 1 ? Color.Red : Color.Green);
                                cb_C_1.ForeColor = (relais[2] == 1 ? Color.Red : Color.Green);
                                cb_C_2.ForeColor = (relais[3] == 1 ? Color.Red : Color.Green);
                                cb_C_3.ForeColor = (relais[4] == 1 ? Color.Red : Color.Green);
                                cb_C_4.ForeColor = (relais[5] == 1 ? Color.Red : Color.Green);
                                cb_C_5.ForeColor = (relais[6] == 1 ? Color.Red : Color.Green);
                                cb_C_6.ForeColor = (relais[7] == 1 ? Color.Red : Color.Green);
                                cb_C_7.ForeColor = (relais[8] == 1 ? Color.Red : Color.Green);
                                cb_C_8.ForeColor = (relais[9] == 1 ? Color.Red : Color.Green);

                                cb_L_1.ForeColor = (relais[10] == 0 ? Color.Red : Color.Green);
                                cb_L_2.ForeColor = (relais[11] == 0 ? Color.Red : Color.Green);
                                cb_L_3.ForeColor = (relais[12] == 0 ? Color.Red : Color.Green);
                                cb_L_4.ForeColor = (relais[13] == 0 ? Color.Red : Color.Green);
                                cb_L_5.ForeColor = (relais[14] == 0 ? Color.Red : Color.Green);
                                cb_L_6.ForeColor = (relais[15] == 0 ? Color.Red : Color.Green);
                                cb_L_7.ForeColor = (relais[16] == 0 ? Color.Red : Color.Green);
                                cb_L_8.ForeColor = (relais[17] == 0 ? Color.Red : Color.Green);
                                cb_L_9.ForeColor = (relais[18] == 0 ? Color.Red : Color.Green);
                            }

                            if (type == 4)
                            {
                                // Infotext
                                String infotext = "";
                                for (int i = 0; i < rxlen; i++)
                                    infotext = infotext + (char)rxarr[i];

                                AppendInfo(infotext);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    serialopen = false;
                    connected = false;
                    //Console.WriteLine("READ ERROR: ", ex.ToString());

                }
            }
            else
            {
                //Console.WriteLine("SER PORT CLOSED");
            }

            if(rp.result != null)
            {
                rpres_L = rp.result.searchL;
                rpres_C = rp.result.searchC;
                rpres_R = rp.result.searchR;
                pictureBox_diagramm.Invalidate();
                rp.result = null;
            }
        }

        const int maxrxlen = 200;
        Byte[] rxdata = new Byte[maxrxlen];
        int rxstat = 0;
        float flastqrg = 0;

        private void showProtocol()
        {
            int qrg = rxarr[0];
            qrg <<= 8;
            qrg += rxarr[1];
            float fqrg = qrg;
            fqrg /= 1000;   // MHz

            int swr = rxarr[2];
            swr <<= 8;
            swr += rxarr[3];
            float fs = swr;
            fs /= 100;

            int L = rxarr[4];
            L <<= 8;
            L += rxarr[5];

            int C = rxarr[6];

            if(Math.Abs(fqrg - flastqrg) > 1 || fqrg < flastqrg)
            {
                AppendInfo("~Tuning Protokoll " + fqrg.ToString() + " MHz", rtb_prot);
                flastqrg = fqrg;
            }

            String red = "";
            if (swr > suchval)
                red = "§";

            String line = fqrg.ToString("0.000").PadLeft(7) + " MHz ... " + red + "1 : " + fs.ToString("0.00").PadLeft(6) +
                              "        C = " + Cval(C) + " pF  L = " + Lval(L) + "uH\r\n";

            AppendInfo(line, rtb_prot);
        }

        String Cval(int c)
        {
            int v = 0;

            if ((c & 1) == 1) v += 25;
            if ((c & 2) == 2) v += 50;
            if ((c & 4) == 4) v += 100;
            if ((c & 8) == 8) v += 200;
            if ((c & 16) == 16) v += 400;
            if ((c & 32) == 32) v += 800;
            if ((c & 64) == 64) v += 1600;
            if ((c & 128) == 128) v += 3200;

            return v.ToString();
        }

        String Lval(int L)
        {
            int v = 0;

            if ((L & 1) == 1) v += 50;
            if ((L & 2) == 2) v += 100;
            if ((L & 4) == 4) v += 200;
            if ((L & 8) == 8) v += 400;
            if ((L & 16) == 16) v += 800;
            if ((L & 32) == 32) v += 1600;
            if ((L & 64) == 64) v += 3200;
            if ((L & 128) == 128) v += 6400;
            if ((L & 256) == 256) v += 12800;

            double dv = v;
            dv /= 1000;

            return dv.ToString("n3");
        }

        private void tb_real_TextChanged(object sender, EventArgs e)
        {
            LCcalc();
        }

        private void tb_imagC_TextChanged(object sender, EventArgs e)
        {
            LCcalc();
        }

        private void tb_imagL_TextChanged(object sender, EventArgs e)
        {
            LCcalc();
        }

        private void tb_freq_TextChanged(object sender, EventArgs e)
        {
            LCcalc();
        }

        private void tb_highpass_CheckedChanged(object sender, EventArgs e)
        {
            LCcalc();
        }

        private void tb_lowpass_CheckedChanged(object sender, EventArgs e)
        {
            LCcalc();
        }

        private void LCcalc()
        {
            double real = MyToDouble(tb_real.Text);
            double c = MyToDouble(tb_imagC.Text);
            double l = MyToDouble(tb_imagL.Text);
            double freq = MyToDouble(tb_freq.Text);
            calcMatch cm = new calcMatch(real, c, l, freq, tb_highpass.Checked);
            LCresult res = cm.calc();
            lb_R.Text = "R = " + res.R.ToString("n2") + " Ohm";
            lb_X.Text = "jX= " + res.X.ToString("n2") + " Ohm";
            lb_Z.Text = "Z = " + res.Z.ToString("n2") + " Ohm";
            lb_rp.Text = "Rp= " + res.Rp.ToString("n2") + " Ohm";
            lb_xp.Text = "Xp = " + res.Xp.ToString("n2") + " Ohm";
            lb_LCresult.Text = "C = " + res.cap_pF.ToString("n0") + " pF , L = " +
                                res.ind_uH.ToString("n2") + " uH , L an ";
            if (res.LtoANT) lb_LCresult.Text += "ANT";
            else lb_LCresult.Text += "TRX";
        }

        // Culture invariant conversion
        public double MyToDouble(String s)
        {
            double r = 0;

            try
            {
                s = s.Replace(',', '.');
                r = Convert.ToDouble(s, System.Globalization.CultureInfo.InvariantCulture);
            }
            catch
            {
            }
            return r;
        }

        private void bt_calctune_Click(object sender, EventArgs e)
        {
            double real = MyToDouble(tb_real.Text);
            double c = MyToDouble(tb_imagC.Text);
            double l = MyToDouble(tb_imagL.Text);
            double freq = MyToDouble(tb_freq.Text);
            calcMatch cm = new calcMatch(real, c, l, freq, tb_highpass.Checked);
            LCresult res = cm.calc();

            if (res.LtoANT)
                sendtuner2("MS", (Byte)1);
            else
                sendtuner2("MS", (Byte)2);

            sendtuner2("X", Convert.ToInt16(res.cap_pF));
            Thread.Sleep(100);
            sendtuner2("Y", Convert.ToInt16(res.ind_uH * 1000));
            Thread.Sleep(100);
            sendtuner2("AS", false);    // Feinabstimmung
        }

        // Revers Berechnung

        private void tb_revC_TextChanged(object sender, EventArgs e)
        {
        }

        private void tb_revL_TextChanged(object sender, EventArgs e)
        {
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
        }

        void calcREV()
        {
            double acc_C = 5;
            double acc_L = 0.05;
            String sres = "";

            tb_revResult.Text = "calculating ...";
            //try
            {
                double c_soll = MyToDouble(tb_revC.Text);
                double l_soll = MyToDouble(tb_revL.Text);
                double freq = MyToDouble(tb_revFreq.Text);

                if (c_soll < 0.1 || l_soll < 0.01 || freq < 0.0001) return;

                for (double real = 1; real < 1000; real += 1)
                {
                    tb_revResult.Text = "scan real: " + real.ToString("n1");
                    Application.DoEvents();

                    for (double c = 1; c < 2000; c += 1)
                    {
                        calcMatch cm = new calcMatch(real, c, 0, freq, tb_revHP.Checked);
                        LCresult res = cm.calc();

                        if (matched(res.cap_pF, c_soll, acc_C) && matched(res.ind_uH, l_soll, acc_L))
                        {
                            sres += "R: " + real.ToString("n1") +
                                    " C: " + c.ToString("n1") + "\r\n";
                            //return;
                        }
                    }

                    for (double l = 0.1; l < 30; l += 0.1)
                    {
                        calcMatch cm = new calcMatch(real, 0, l, freq, tb_revHP.Checked);
                        LCresult res = cm.calc();

                        if (matched(res.ind_uH, l_soll, acc_L) && matched(res.cap_pF, c_soll, acc_C))
                        {
                            sres += "R: " + real.ToString("n1") +
                                                " L: " + l.ToString("n1") + "\r\n";
                            //return;
                        }
                    }
                }
            }
            //catch
            {
                //tb_revResult.Text = "err";
            }
            tb_revResult.Text = sres;
        }

        bool matched(double vist, double vsoll, double acc)
        {
            if (vist >= (vsoll - acc) && vist <= (vsoll + acc)) return true;
            return false;
        }

        private void bt_revcalc_Click(object sender, EventArgs e)
        {
            calcREV();
        }

        private void cb_RXTX_CheckedChanged(object sender, EventArgs e)
        {
            sendtuner2("TT", cb_RXTX.Checked);
        }

        private void cb_FM_CheckedChanged(object sender, EventArgs e)
        {
            sendtuner2("TM", cb_FM.Checked);
        }

        private Byte [] FreqToBCD(UInt32 freq)
        {
            Byte[] farr = new Byte[5];

            UInt32 _10MHz = freq / 10000000;
            freq -= (_10MHz * 10000000);

            UInt32 _1MHz = freq / 1000000;
            freq -= (_1MHz * 1000000);

            UInt32 _100kHz = freq / 100000;
            freq -= (_100kHz * 100000);

            UInt32 _10kHz = freq / 10000;
            freq -= (_10kHz * 10000);

            UInt32 _1kHz = freq / 1000;
            freq -= (_1kHz * 1000);

            UInt32 _100Hz = freq / 100;
            freq -= (_100Hz * 100);

            UInt32 _10Hz = freq / 10;
            freq -= (_10Hz * 10);

            UInt32 _1Hz = freq;

            farr[0] = (Byte)((Byte)(_10Hz << 4) + (Byte)_1Hz);
            farr[1] = (Byte)((Byte)(_1kHz << 4) + (Byte)_100Hz);
            farr[2] = (Byte)((Byte)(_100kHz << 4) + (Byte)_10kHz);
            farr[3] = (Byte)((Byte)(_10MHz << 4) + (Byte)_1MHz);
            farr[4] = 0;

            return farr;
        }

        private void button_setqrg_Click(object sender, EventArgs e)
        {
            try
            {
                UInt32 freq_Hz = Convert.ToUInt32(1000000 * MyToDouble(tb_mhz.Text));
                Byte[] freq = FreqToBCD(freq_Hz);
                sendtuner2("TF", freq);
            }
            catch { }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            UInt32 f = civ_freq - 10000;
            Double uf = (double)f / 1000000.0;
            tb_mhz.Text = uf.ToString();
            Byte[] freq = FreqToBCD(f);
            sendtuner2("TF", freq);
        }

        private void button6_Click(object sender, EventArgs e)
        {
            UInt32 f = civ_freq + 10000;
            Double uf = (double)f / 1000000.0;
            tb_mhz.Text = uf.ToString();
            Byte[] freq = FreqToBCD(f);
            sendtuner2("TF", freq);
        }

        private void bt_clearact_Click(object sender, EventArgs e)
        {
            UInt32 f = civ_freq;
            Double uf = (double)f / 1000000.0;
            tb_mhz.Text = uf.ToString();
            Byte[] freq = FreqToBCD(f);
            sendtuner2("TL", freq);
        }

        private void cb_C_minus_Click(object sender, EventArgs e)
        {
            Byte v = 0;
            if (cb_C_1.ForeColor == Color.Red) v |= 1;
            if (cb_C_2.ForeColor == Color.Red) v |= 2;
            if (cb_C_3.ForeColor == Color.Red) v |= 4;
            if (cb_C_4.ForeColor == Color.Red) v |= 8;
            if (cb_C_5.ForeColor == Color.Red) v |= 16;
            if (cb_C_6.ForeColor == Color.Red) v |= 32;
            if (cb_C_7.ForeColor == Color.Red) v |= 64;
            if (cb_C_8.ForeColor == Color.Red) v |= 128;

            if (v > 0) v--;

            sendtuner2("CV", v);
        }

        private void cb_C_plus_Click(object sender, EventArgs e)
        {
            Byte v = 0;
            if (cb_C_1.ForeColor == Color.Red) v |= 1;
            if (cb_C_2.ForeColor == Color.Red) v |= 2;
            if (cb_C_3.ForeColor == Color.Red) v |= 4;
            if (cb_C_4.ForeColor == Color.Red) v |= 8;
            if (cb_C_5.ForeColor == Color.Red) v |= 16;
            if (cb_C_6.ForeColor == Color.Red) v |= 32;
            if (cb_C_7.ForeColor == Color.Red) v |= 64;
            if (cb_C_8.ForeColor == Color.Red) v |= 128;

            if (v < 255) v++;

            sendtuner2("CV", v);
        }

        private void bt_grundstellung_Click(object sender, EventArgs e)
        {
            sendtuner2("AG", false);
        }

        private void bt_seek50_Click(object sender, EventArgs e)
        {
            sendtuner2("A5", false);
        }

        private void bt_seekPhi0C_Click(object sender, EventArgs e)
        {
            sendtuner2("A0", false);
        }

        private void bt_swrfein_Click(object sender, EventArgs e)
        {
            sendtuner2("AS", false);
        }

        private void bt_seek_complete_Click(object sender, EventArgs e)
        {
            cb_swr_suche_SelectedIndexChanged(null, null);
            Thread.Sleep(100);
            sendtuner2("AA", false);
            rtb_monitor.Clear();
        }

        private void bt_tuneband_Click(object sender, EventArgs e)
        {
            cb_swr_suche_SelectedIndexChanged(null, null);
            Thread.Sleep(100);
            sendtuner2("AM", false);
            rtb_monitor.Clear();
        }

        private void bt_tune_stop_Click(object sender, EventArgs e)
        {
            sendtuner2("AL", false);
        }

        private void bt_clear_band_mem_Click(object sender, EventArgs e)
        {
            sendtuner2("EB", false);
        }

        private void bt_clear_full_mem_Click(object sender, EventArgs e)
        {
            sendtuner2("EF", false);
        }

        private void cb_swr_suche_SelectedIndexChanged(object sender, EventArgs e)
        {
            String s = cb_swr_suche.Text;
            try
            {
                suchval = (int)(MyToDouble(s) * 100);
            }
            catch
            {
                suchval = 0;
            }
            sendtuner2("W", suchval);
        }

        private void cb_speed_SelectedIndexChanged(object sender, EventArgs e)
        {
            String s = cb_speed.Text;
            int speed = 25;
            try
            {
                speed = (int)MyToDouble(s);
            }
            catch
            {
                speed = 25;
            }
            sendtuner2("R", speed);
        }

        private void bt_test_Click(object sender, EventArgs e)
        {
            sendtuner2("AX", false);
        }

        private void bt_cal_1_Click(object sender, EventArgs e)
        {
            sendtuner2("KE", false);
        }

        private void bt_call_100_Click(object sender, EventArgs e)
        {
            sendtuner2("KH", false);
        }

        private void rb_ant1_CheckedChanged(object sender, EventArgs e)
        {
            if (rb_ant1.Checked) sendtuner2("EA", (Byte)0);
            if (rb_ant2.Checked) sendtuner2("EA", (Byte)1);
            if (rb_ant3.Checked) sendtuner2("EA", (Byte)2);
            if (rb_ant4.Checked) sendtuner2("EA", (Byte)3);
            if (rb_ant5.Checked) sendtuner2("EA", (Byte)4);
            if (rb_ant6.Checked) sendtuner2("EA", (Byte)5);
        }

        private void bt_rtfclear_Click(object sender, EventArgs e)
        {
            rtb_monitor.Clear();
        }

        private void tb_civadr_TextChanged(object sender, EventArgs e)
        {
            Byte civ = 0;

            try
            {
                String s = tb_civadr.Text;
                s = s.Trim();

                if(s.Length == 2)
                {
                    civ = Convert.ToByte(Convert.ToInt32(s.Substring(0, 1)) * 16 + Convert.ToInt32(s.Substring(1, 1)));
                    sendtuner2("KC", civ);
                    tb_civadr.ForeColor = Color.Black;
                }
                else
                    tb_civadr.ForeColor = Color.Red;
            }
            catch
            {
                tb_civadr.ForeColor = Color.Red;
            }
        }

        bool civ_has_focus = false;
        private void tb_civadr_Enter(object sender, EventArgs e)
        {
            civ_has_focus = true;
        }

        private void tb_civadr_Leave(object sender, EventArgs e)
        {
            civ_has_focus = false;
        }

        Byte outpwr = 0;
        private void trackbar_pwr_Scroll(object sender, EventArgs e)
        {
            outpwr = (Byte)trackbar_pwr.Value;
        }

        Byte oldpwr = 0;
        private void timer_control_Tick(object sender, EventArgs e)
        {
            if(outpwr != oldpwr)
            {
                oldpwr = outpwr;
                sendtuner2("A1", (Byte)outpwr);
            }
            readcheck++;
        }

        private void button4_Click(object sender, EventArgs e)
        {
            rtb_prot.Clear();
        }

        private void bt_seekZ50C_Click(object sender, EventArgs e)
        {
            sendtuner2("AY", false);
        }

        private void button7_Click(object sender, EventArgs e)
        {
            sendtuner2("ES", false);
        }

        private void tb_mhz_Enter(object sender, EventArgs e)
        {
            tbmhz_hasfocus = 99;
        }

        private void tb_mhz_Leave(object sender, EventArgs e)
        {
            tbmhz_hasfocus = 5;
        }

        private void bt_phiplusL_Click(object sender, EventArgs e)
        {
            sendtuner2("AZ", false);
        }

        private void pictureBox_diagramm_Paint(object sender, PaintEventArgs e)
        {
            double Z = rpres_R;
            String LC_Einheit = "uH";
            String stellen = "n3";
            double LC = rpres_L;
            if (LC < 0.001)
            {
                LC = rpres_C;
                LC_Einheit = "pF";
                stellen = "n0";
            }

            double C = cap*2;
            double L = ind;
            String ec = "pF";
            String el = "nH";

            if(cap >=1000)
            {
                C /= 1000;
                ec = "nF";
            }

            if(ind >= 1000)
            {
                L /= 1000;
                el = "uH";
            }


            using (Font fnt = new Font("Arial", 12))
            {
                e.Graphics.DrawString(C.ToString("n3") + ec, fnt, Brushes.Blue, new Point(145, 50));
                e.Graphics.DrawString(C.ToString("n3") + ec, fnt, Brushes.Blue, new Point(145, 327));
                e.Graphics.DrawString("TRX", fnt, Brushes.Blue, new Point(1, 225));
                e.Graphics.DrawString("50 Ohm", fnt, Brushes.Blue, new Point(1, 245));

                // vertikale Texte
                StringFormat sf = new StringFormat { FormatFlags = StringFormatFlags.DirectionVertical };
                e.Graphics.DrawString(L.ToString("n3") + el, fnt, Brushes.Blue, new Point(180, 160),sf);
                e.Graphics.DrawString("Antenne", fnt, Brushes.Blue, new Point(295, 165),sf);
                if (Z < 99999)
                {
                    e.Graphics.DrawString(Z.ToString("n0") + " Ohm", fnt, Brushes.Blue, new Point(270, 110), sf);
                    e.Graphics.DrawString(LC.ToString(stellen) + " " + LC_Einheit, fnt, Brushes.Blue, new Point(270, 230), sf);
                }
            }

            // Relaisstrich
            if(anttrx)
            {
                e.Graphics.DrawLine(new Pen(Color.FromArgb(0xa5,0x4b,0x4b), 4), new Point(174, 150), new Point(183, 139));
                e.Graphics.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 4), new Point(174, 247), new Point(183, 256));
            }
            else
            {
                e.Graphics.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 4), new Point(174, 150), new Point(163, 139));
                e.Graphics.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 4), new Point(174, 247), new Point(163, 256));
            }
            rxing = 0;
        }

        // Positions der Pfeile
        static int midpos = 199;
        static int yoff = 8;
        static int x1left = 240;
        static int arrwid = 6;
        static int xspace = 8;

        PointF[] arr1 = { new PointF(x1left, midpos + yoff), new PointF(x1left + arrwid, midpos), new PointF(x1left, midpos - yoff) };
        PointF[] arr2 = { new PointF(x1left + xspace, midpos + yoff), new PointF(x1left + xspace + arrwid, midpos), new PointF(x1left + xspace, midpos - yoff) };
        PointF[] arr3 = { new PointF(x1left + xspace * 2, midpos + yoff), new PointF(x1left + xspace * 2 + arrwid, midpos), new PointF(x1left + xspace * 2, midpos - yoff) };
        PointF[] arr4 = { new PointF(x1left + xspace * 3, midpos + yoff), new PointF(x1left + xspace * 3 + arrwid, midpos), new PointF(x1left + xspace * 3, midpos - yoff) };
        PointF[] arr5 = { new PointF(x1left + xspace * 4, midpos + yoff), new PointF(x1left + xspace * 4 + arrwid, midpos), new PointF(x1left + xspace * 4, midpos - yoff) };

        PointF[] arr3rx = { new PointF(x1left + xspace * 2 + arrwid, midpos + yoff), new PointF(x1left + xspace * 2 , midpos), new PointF(x1left + xspace * 2 + arrwid, midpos - yoff) };
        int rxing = -1;

        void makeDiagArrow(int phase)
        {
            Graphics g = pictureBox_diagramm.CreateGraphics();
            int yo = 193;
            int yu = 206;

            if(iswr == 0)
            {
                if (rxing != 1)
                {
                    // RX
                    g.DrawPolygon(new Pen(Color.White, 3), arr1);
                    g.DrawPolygon(new Pen(Color.White, 3), arr2);
                    g.DrawPolygon(new Pen(Color.White, 3), arr3);
                    g.DrawPolygon(new Pen(Color.White, 3), arr4);
                    g.DrawPolygon(new Pen(Color.White, 3), arr5);
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yo), new Point(290, yo));
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yu), new Point(290, yu));
                    g.DrawPolygon(new Pen(Color.DarkGreen, 3), arr3rx);
                }
                rxing = 1;
                return;
            }

            rxing = 0;
            g.DrawPolygon(new Pen(Color.White, 3), arr3rx);

            switch (phase)
            {
                case 0:
                    g.DrawPolygon(new Pen(Color.White, 3), arr2);
                    g.DrawPolygon(new Pen(Color.White, 3), arr3);
                    g.DrawPolygon(new Pen(Color.White, 3), arr4);
                    g.DrawPolygon(new Pen(Color.White, 3), arr5);
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yo), new Point(290, yo));
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yu), new Point(290, yu));
                    g.DrawPolygon(new Pen(Color.Red, 3), arr1);
                    break;
                case 1:
                    g.DrawPolygon(new Pen(Color.White, 3), arr1);
                    g.DrawPolygon(new Pen(Color.White, 3), arr3);
                    g.DrawPolygon(new Pen(Color.White, 3), arr4);
                    g.DrawPolygon(new Pen(Color.White, 3), arr5);
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yo), new Point(290, yo));
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yu), new Point(290, yu));
                    g.DrawPolygon(new Pen(Color.Red, 3), arr2);
                    break;
                case 2:
                    g.DrawPolygon(new Pen(Color.White, 3), arr1);
                    g.DrawPolygon(new Pen(Color.White, 3), arr2);
                    g.DrawPolygon(new Pen(Color.White, 3), arr4);
                    g.DrawPolygon(new Pen(Color.White, 3), arr5);
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yo), new Point(290, yo));
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yu), new Point(290, yu));
                    g.DrawPolygon(new Pen(Color.Red, 3), arr3);
                    break;
                case 3:
                    g.DrawPolygon(new Pen(Color.White, 3), arr1);
                    g.DrawPolygon(new Pen(Color.White, 3), arr2);
                    g.DrawPolygon(new Pen(Color.White, 3), arr3);
                    g.DrawPolygon(new Pen(Color.White, 3), arr5);
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yo), new Point(290, yo));
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yu), new Point(290, yu));
                    g.DrawPolygon(new Pen(Color.Red, 3), arr4);
                    break;
                case 4:
                    g.DrawPolygon(new Pen(Color.White, 3), arr1);
                    g.DrawPolygon(new Pen(Color.White, 3), arr2);
                    g.DrawPolygon(new Pen(Color.White, 3), arr3);
                    g.DrawPolygon(new Pen(Color.White, 3), arr4);
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yo), new Point(290, yo));
                    g.DrawLine(new Pen(Color.FromArgb(0xa5, 0x4b, 0x4b), 3), new Point(224, yu), new Point(290, yu));
                    g.DrawPolygon(new Pen(Color.Red, 3), arr5);
                    break;
            }
        }

        int arrphase = 0;
        private void timer_arrows_Tick(object sender, EventArgs e)
        {
            makeDiagArrow(arrphase);
            if (++arrphase == 5)
                arrphase = 0;
        }

        int rxlen, rxcnt, rxcrc16, crc16;

        int got = 0;
        Byte[] rxarr = null;
        int type = 0;
        private void DataRec(int ret)
        {
            {
                switch (rxstat)
                {
                    case 0:
                        rxdata[rxstat] = (Byte)ret;
                        if (ret == 0x32)
                        {
                            rxstat++;
                        }
                        break;
                    case 1:
                        rxdata[rxstat] = (Byte)ret;
                        if (ret == 0x1c)
                            rxstat++;
                        else
                            rxstat = 0;
                        break;
                    case 2:
                        rxdata[rxstat] = (Byte)ret;
                        if (ret == 0xab)
                            rxstat++;
                        else
                            rxstat = 0;
                        break;
                    case 3:
                        type = rxdata[rxstat] = (Byte)ret;
                        rxstat++;
                        break;
                    case 4:
                        rxdata[rxstat] = (Byte)ret;
                        rxlen = ret;
                        rxlen <<= 8;
                        rxstat++;
                        break;
                    case 5:
                        rxdata[rxstat] = (Byte)ret;
                        rxlen += ret;
                        rxcnt = 0;
                        rxstat++;
                        break;
                    case 6:
                        if (rxcnt >= maxrxlen)
                        {
                            rxstat = 0;
                            break;
                        }
                        rxdata[6 + rxcnt++] = (Byte)ret;
                        if (rxcnt == rxlen)
                        {
                            rxstat++;
                        }
                        break;
                    case 7:
                        rxcrc16 = ret;
                        rxcrc16 <<= 8;
                        rxstat++;
                        break;
                    case 8:
                        rxcrc16 += ret;
                        crc16 = crc16_messagecalc(rxdata, rxlen+6);
                        if (crc16 == rxcrc16)
                        {
                            got = 1;
                            rxarr = new Byte[rxlen];
                            for (int i = 0; i < rxlen; i++)
                            {
                                rxarr[i] = rxdata[i + 6];
                            }
                        }
                        rxstat = 0;
                        break;
                }
            }
        }

        void AppendInfo(String s1, RichTextBox box = null)
        {
            Color col = Color.Black;
            Font fnt = new Font("Verdana", (float)8.25);

            if (s1.Length < 1) return;

            if(s1.Substring(0,1) == "#")
            {
                //rtb_monitor.Clear();
                s1 = s1.Substring(1, s1.Length - 1);
            }

            if (s1.Substring(0, 1) == "~")
            {
                s1 = s1.Substring(1, s1.Length - 1);
                fnt = new Font("Verdana", (float)8.25, FontStyle.Bold);
            }

            String s = s1.Trim(new char[] { '\n', '\r' });
            if (s == null || s.Length <= 1) return;
            s += "\n";

            if (s.Contains("§"))
            {
                String[] sa = s.Split(new char[] { '§' });
                foreach(String sx in sa)
                {
                    AppendTextOnce(fnt, col, Color.White, sx, box);
                    col = Color.Red;
                }
                return;
            }

            AppendTextOnce(fnt , col, Color.White, s, box);
        }

        private void AppendTextOnce(Font selfont, Color color, Color bcolor, string text, RichTextBox box = null)
        {
            try
            {
                if (box == null)
                    box = rtb_monitor;

                // max. xxx Zeilen, wenn mehr dann lösche älteste
                if (box.Lines.Length > 5000)
                {
                    box.SelectionStart = 0;
                    box.SelectionLength = box.Text.IndexOf("\n", 0) + 1;
                    box.SelectedText = "";
                }

                int start = box.TextLength;
                box.AppendText(text);
                int end = box.TextLength;

                // Textbox may transform chars, so (end-start) != text.Length
                box.Select(start, end - start);
                box.SelectionColor = color;
                box.SelectionFont = selfont;
                box.SelectionBackColor = bcolor;
                box.Select(end, 0);

                box.ScrollToCaret();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }
    }

    public class LCresult
    {
        public double R, L, C, X, Z, Rp, Xp;
        public double cap_pF, ind_uH;
        public bool LtoANT;
        public double searchC, searchL, searchR;
        public double diff;
    }
}
